//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

// Using shapes, the Chart1 duplicates a TTreeView tree of nodes.
// The shapes can be selected, double-clicked, etc.

#include "Unit1.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "TeeComma"
#pragma resource "*.dfm"
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
  : TForm(Owner)
{
}
//---------------------------------------------------------------------------

// When a shape is clicked, set TreeView.Selected node
void TForm1::CheckSelected()
{
  if (Selected != 0)
    TreeView1->Selected = reinterpret_cast<TTreeNode*>(Selected->Tag);
  Chart1->Repaint();
}

// draw lines to the selected shape child shapes 
void TForm1::DrawSeriesLines(TChartShape* ASeries)
{
  int tmpX;
  TTreeNode* node = reinterpret_cast<TTreeNode*>(ASeries->Tag);
  if (node->Expanded)
    for (int i=0;i<node->Count;i++) {
      Chart1->Canvas->MoveTo3D(ASeries->Bounds.Left,ASeries->Bounds.Bottom,ASeries->MiddleZ);
        if (SquaredLine)
          tmpX = ASeries->Bounds.Left;
      TChartShape* cs = ShapeOf(node->Item[i]);
      if (SquaredLine)
        Chart1->Canvas->LineTo3D(tmpX,
          (cs->Bounds.Top + cs->Bounds.Bottom) / 2, cs->MiddleZ);
      Chart1->Canvas->LineTo3D(cs->Bounds.Left,
        (cs->Bounds.Top + cs->Bounds.Bottom) / 2, cs->MiddleZ);
    }
}

void __fastcall TForm1::AfterDrawSeries(TObject* Sender)
{
  Chart1->Canvas->Pen->Style = psSolid;
  Chart1->Canvas->Pen->Color = clSilver;
  Chart1->Canvas->Pen->Width = 1;
  Chart1->Canvas->Brush->Style = bsClear;
  TChartShape* cs = dynamic_cast<TChartShape*>(Sender);
  DrawSeriesLines(cs);
}

// Fill a Chart with shapes created from a TTreeView tree
void TForm1::CreateShapes(TTreeNode* Node, int Delta)
{
  int PosX, PosY;
  TChartShape* Shape = new TChartShape(this);
  Shape->Tag = Longint(Node);
  Shape->ParentChart = Chart1;
  Shape->SeriesColor = GetDefaultColor(Shape->ParentChart->SeriesCount());
  Shape->Title = Node->Text;
  Shape->Text->Add(Shape->Title);
  Shape->Style = chasCube;
  Shape->Pen->Color = clSilver;
  Shape->Font->Size = 14;
  if (Shape->SeriesColor != clYellow)
    Shape->Font->Color = clWhite;
  if (Shape->SeriesColor == clWhite)
    Shape->Font->Color = clBlack;
  if (Shape->ParentChart->SeriesCount() == 1) {
    PosX = 0;
    PosY = 0;
  }
  else {
    PosX = Delta;
    PosY =  -(Shape->ParentChart->SeriesCount() - 1) * 60;
  }
  Shape->X0 = PosX-50;
  Shape->X1 = PosX + 50;
  Shape->Y0 = PosY-25;
  Shape->Y1 = PosY + 25;
  Shape->AfterDrawValues = AfterDrawSeries;
  for (int i=0;i<Node->Count;i++)
    CreateShapes(Node->Item[i], Delta + 20);
}

// Sets the shape Series Active property according to "Node"
void TForm1::ExpandCollapseShapes(TTreeNode* Node, bool IsExpand)
{
  if (Chart1->SeriesCount() > 0)
    for (int i=0;i<Node->Count;i++) {
      TChartShape* cs = ShapeOf(Node->Item[i]);
      cs->Active = IsExpand;
      ExpandCollapseShapes(Node->Item[i], IsExpand);
    }
}

// Returns the shape Series corresponding to a TreeView node
TChartShape* TForm1::ShapeOf(TTreeNode* Node)
{
  for (int i=0;i<Chart1->SeriesCount();i++)
    if (Chart1->Series[i]->Tag == (int)Node)
      return dynamic_cast<TChartShape*>(Chart1->Series[i]);
  return 0;
}

// Initialize several chart properties and create shapes
void __fastcall TForm1::FormCreate(TObject *Sender)
{
  Selected = 0;
  SquaredLine = false;
  Chart1->AxisVisible = false;
  Chart1->ClipPoints = false;
  Chart1->View3DOptions->Orthogonal = false;
  Chart1->View3DOptions->Rotation = 348;
  Chart1->View3DOptions->Elevation = 335;
  Chart1->View3DOptions->Zoom = 88;
  TeeCommander1->ButtonRotate->Down = true;
  TreeView1->Items->Item[0]->Expand(true);
  CreateShapes(TreeView1->Items->Item[0],20);
}
//---------------------------------------------------------------------------
// When TreeView is collapsed, change shapes
void __fastcall TForm1::TreeView1Collapsed(TObject *Sender,
      TTreeNode *Node)
{
  ExpandCollapseShapes(Node, false);
}
//---------------------------------------------------------------------------
// When TreeView is expanded, change shapes
void __fastcall TForm1::TreeView1Expanded(TObject *Sender, TTreeNode *Node)
{
  ExpandCollapseShapes(Node, true);
}
//---------------------------------------------------------------------------
// When a shape is clicked, set TreeView.Selected node
void __fastcall TForm1::Chart1ClickSeries(TCustomChart *Sender,
      TChartSeries *Series, int ValueIndex, TMouseButton Button,
      TShiftState Shift, int X, int Y)
{
  Selected = dynamic_cast<TChartShape*>(Series);
  CheckSelected();
}
//---------------------------------------------------------------------------
// Paint a rectangle around the selected shape,
//  and draw lines to the selected shape child shapes
void __fastcall TForm1::Chart1AfterDraw(TObject *Sender)
{
  if (Selected != 0) {
    Chart1->Canvas->Pen->Style = psSolid;
    Chart1->Canvas->Pen->Color = clRed;
    Chart1->Canvas->Pen->Width = 2;
    Chart1->Canvas->Brush->Style = bsClear;
    Chart1->Canvas->Cube(Selected->Bounds.Left, Selected->Bounds.Right, Selected->Bounds.Top,
      Selected->Bounds.Bottom, Selected->StartZ, Selected->EndZ, true);
    DrawSeriesLines(Selected);
  }
}
//---------------------------------------------------------------------------
// Unselect
void __fastcall TForm1::Chart1ClickBackground(TCustomChart *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
  if (Selected != 0) {
    TreeView1->Selected = 0;
    Selected = 0;
    Chart1->Repaint();
  }
  Chart1->CancelMouse = false;
}
//---------------------------------------------------------------------------
// Unselect
void __fastcall TForm1::TreeView1Click(TObject *Sender)
{
  Selected = ShapeOf(TreeView1->Selected);
  Chart1->Repaint();
}
//---------------------------------------------------------------------------
// Set 2D / 3D
void __fastcall TForm1::CheckBox1Click(TObject *Sender)
{
  CheckBox2->Enabled = !CheckBox1->Checked;
  Chart1->View3D = !CheckBox1->Checked;
}
//---------------------------------------------------------------------------
// Apply Z order
void __fastcall TForm1::CheckBox2Click(TObject *Sender)
{
  Chart1->ApplyZOrder = CheckBox2->Checked;
  Chart1->Repaint();
}
//---------------------------------------------------------------------------
// Change the shape text when a Tree node text is changed
void __fastcall TForm1::TreeView1Edited(TObject *Sender, TTreeNode *Node,
      AnsiString &S)
{
  TChartShape* cs = ShapeOf(Node);
  cs->Clear();
  cs->Text->Add(S);
}
//---------------------------------------------------------------------------
// Show / hide legend
void __fastcall TForm1::CheckBox4Click(TObject *Sender)
{
  Chart1->Legend->Visible = CheckBox4->Checked;
}
//---------------------------------------------------------------------------
// Show / hide 3D walls
void __fastcall TForm1::CheckBox3Click(TObject *Sender)
{
  Chart1->View3DWalls = CheckBox3->Checked;
  if (Chart1->View3DWalls)
    Chart1->BackWall->Brush->Style = bsSolid;
  else
    Chart1->BackWall->Brush->Style = bsClear;
  Chart1->BackWall->Pen->Visible = Chart1->View3DWalls;
}
//---------------------------------------------------------------------------
// Expand or collapse a Tree node when a shape is double-clicked
void __fastcall TForm1::Chart1DblClick(TObject *Sender)
{
  TChartClickedPart Part;
  Chart1->CalcClickedPart(Chart1->GetCursorPos(), Part);
  if (Part.Part == cpSeries) {
    Selected =  dynamic_cast<TChartShape*>(Part.ASeries);
    TTreeNode* node = reinterpret_cast<TTreeNode*>(Part.ASeries->Tag);
    if (node->Expanded)
      node->Collapse(true);
    else
      node->Expand(true);
    CheckSelected();
  }
}
//---------------------------------------------------------------------------
// Show / hide axes
void __fastcall TForm1::CheckBox5Click(TObject *Sender)
{
  Chart1->AxisVisible = CheckBox5->Checked;
}
//---------------------------------------------------------------------------
// Draw lines from parent shapes to child shapes
void __fastcall TForm1::CheckBox6Click(TObject *Sender)
{
  SquaredLine = CheckBox6->Checked;
  Chart1->Repaint();
}
//---------------------------------------------------------------------------
